@@ -29,6 +29,8 @@
 #include <config.h>
 #include "nautilus-dnd.h"
 
+#include "nautilus-program-choosing.h"
+#include "nautilus-link.h"
 #include <eel/eel-glib-extensions.h>
 #include <eel/eel-string.h>
 #include <eel/eel-vfs-extensions.h>
@@ -45,8 +47,6 @@
 #include <stdio.h>
 #include <string.h>
 
-#define COMMAND_SPECIFIER "command:"
-
 /* a set of defines stolen from the eel-icon-dnd.c file.
  * These are in microseconds.
  */
@@ -278,10 +278,14 @@ nautilus_drag_default_drop_action_for_icons (GdkDragContext *context,
 		return;
 	}
 	
-	/* Check for trash URI.  We do a find_directory for any Trash directory. */
+	/*
+	 * Check for trash URI.  We do a find_directory for any Trash directory.
+	 * Passing 0 permissions as gnome-vfs would override the permissions
+	 * passed with 700 while creating .Trash directory
+	 */
 	if (eel_uri_is_trash (target_uri_string)) {
 		result = gnome_vfs_find_directory (NULL, GNOME_VFS_DIRECTORY_KIND_TRASH,
-						   &target_uri, TRUE, FALSE, 0777);
+						   &target_uri, TRUE, FALSE, 0);
 		if (result != GNOME_VFS_OK) {
 			*action = 0;
 			return;
@@ -293,7 +297,8 @@ nautilus_drag_default_drop_action_for_icons (GdkDragContext *context,
 		}
 		return;
 
-	} else if (eel_str_has_prefix (target_uri_string, COMMAND_SPECIFIER)) {
+	} else if (eel_str_has_prefix (target_uri_string, NAUTILUS_COMMAND_SPECIFIER)
+			|| eel_str_has_prefix (target_uri_string, NAUTILUS_DESKTOP_COMMAND_SPECIFIER)) {
 		if (actions & GDK_ACTION_MOVE) {
 			*action = GDK_ACTION_MOVE;
 		}
@@ -727,3 +732,30 @@ nautilus_drag_autoscroll_stop (NautilusDragInfo *drag_info)
 		drag_info->auto_scroll_timeout_id = 0;
 	}
 }
+
+gboolean
+nautilus_drag_selection_includes_special_link (GList *selection_list)
+{
+	GList *node;
+	char *uri, *local_path;
+	gboolean link_in_selection;
+
+	link_in_selection = FALSE;
+
+	for (node = selection_list; node != NULL; node = node->next) {
+		uri = ((NautilusDragSelectionItem *) node->data)->uri;
+
+		/* FIXME bugzilla.gnome.org 43020: This does sync. I/O and works only locally. */
+		local_path = gnome_vfs_get_local_path_from_uri (uri);
+		link_in_selection = local_path != NULL
+			&& (nautilus_link_local_is_trash_link (local_path) || nautilus_link_local_is_home_link (local_path) ||
+			nautilus_link_local_is_volume_link (local_path));
+		g_free (local_path);
+		
+		if (link_in_selection) {
+			break;
+		}
+	}
+	
+	return link_in_selection;
+}
