@@ -56,15 +56,15 @@ make_full_icon_path (const char *path,
 	
 	partial_path = g_strconcat (path, suffix, NULL);
 
-	if (path[0] == '/') {
+	if (path[0] == '/' && g_file_test (partial_path, G_FILE_TEST_EXISTS)) {
 		return partial_path;
 	}
 
 	/* Build a path for this icon, depending on the theme_is_in_user_directory boolean. */
 	if (theme_is_in_user_directory) {
 		user_directory = nautilus_get_user_directory ();
-		themes_directory = nautilus_make_path (user_directory, "themes");
-		full_path = nautilus_make_path (themes_directory, partial_path);
+		themes_directory = g_build_filename (user_directory, "themes", NULL);
+		full_path = g_build_filename (themes_directory, partial_path, NULL);
 		g_free (user_directory);
 		g_free (themes_directory);
 
@@ -73,7 +73,7 @@ make_full_icon_path (const char *path,
 			full_path = NULL;
 		}
 	} else if (document_type_icon) {
-		full_path = nautilus_make_path (DATADIR "/pixmaps/document-icons/", partial_path);
+		full_path = g_build_filename (DATADIR "/pixmaps/document-icons/", partial_path, NULL);
 		if (!g_file_test (full_path, G_FILE_TEST_EXISTS)) {
 			g_free (full_path);
 			full_path = NULL;
@@ -125,29 +125,17 @@ parse_attach_points (NautilusEmblemAttachPoints *attach_points,
 }
 
 static void
-read_details (const char *path,
-	      guint icon_size,
-	      NautilusIconDetails *details)
+details_from_doc_node (xmlDocPtr doc,
+		       const char *size_as_string,
+		       NautilusIconDetails *details)
 {
-	xmlDocPtr doc;
 	xmlNodePtr node;
-	char *size_as_string, *property;
+	char c, *property;
 	ArtIRect parsed_rect;
-	char c;
 
-	memset (&details->text_rect, 0, sizeof (details->text_rect));
-	
-	if (path == NULL) {
-		return;
-	}
-
-	doc = xmlParseFile (path);
-	
-	size_as_string = g_strdup_printf ("%u", icon_size);
 	node = eel_xml_get_root_child_by_name_and_property
 		(doc, "icon", "size", size_as_string);
-	g_free (size_as_string);
-	
+
 	property = xmlGetProp (node, "embedded_text_rectangle");
 	
 	if (property != NULL) {
@@ -164,10 +152,39 @@ read_details (const char *path,
 	}
 	
 	property = xmlGetProp (node, "attach_points");
-	
 	parse_attach_points (&details->attach_points, property);	
 	xmlFree (property);
+}
+
+static void
+read_details (const char *path,
+	      guint icon_size,
+	      NautilusIconDetails *details,
+	      NautilusIconDetails *opt_scalable_details)
+{
+	xmlDocPtr doc;
+	char *size_as_string;
 	
+	memset (&details->text_rect, 0, sizeof (details->text_rect));
+	if (opt_scalable_details) {
+		memset (&opt_scalable_details->text_rect, 0,
+			sizeof (opt_scalable_details->text_rect));
+	}
+
+	if (path == NULL) {
+		return;
+	}
+
+	doc = xmlParseFile (path);
+
+	size_as_string = g_strdup_printf ("%u", icon_size);
+	details_from_doc_node (doc, size_as_string, details);
+	g_free (size_as_string);
+
+	if (opt_scalable_details) {
+		details_from_doc_node (doc, "*", opt_scalable_details);
+	}
+
 	xmlFreeDoc (doc);
 }
 
@@ -178,7 +195,8 @@ static char *
 get_themed_icon_file_path (const NautilusIconTheme *icon_theme,
 			   const char *icon_name,
 			   guint icon_size,
-			   NautilusIconDetails *details)
+			   NautilusIconDetails *details,
+			   NautilusIconDetails *scalable_details)
 {
 	guint i;
 	gboolean include_size, in_user_directory, document_type_icon;
@@ -235,7 +253,7 @@ get_themed_icon_file_path (const NautilusIconTheme *icon_theme,
 						".xml",
 						in_user_directory,
 						document_type_icon);
-		read_details (xml_path, icon_size, details);
+		read_details (xml_path, icon_size, details, scalable_details);
 		g_free (xml_path);
 	}
 
@@ -256,7 +274,7 @@ theme_has_icon (const NautilusIconTheme *theme,
 
 	path = get_themed_icon_file_path (theme, name,
 					  NAUTILUS_ICON_SIZE_STANDARD,
-					  NULL);
+					  NULL, NULL);
 	g_free (path);
 
 	return path != NULL;
@@ -274,12 +292,12 @@ choose_theme (const NautilusIconThemeSpecifications *theme_specs,
 		return NULL;
 	}
 
-	if (theme_has_icon (&theme_specs->current, name)) {
-		return &theme_specs->current;
+	if (theme_has_icon (theme_specs->current, name)) {
+		return theme_specs->current;
 	}
 
-	if (theme_has_icon (&theme_specs->fallback, name)) {
-		return &theme_specs->fallback;
+	if (theme_has_icon (theme_specs->fallback, name)) {
+		return theme_specs->fallback;
 	}
 
 	return NULL;
@@ -330,7 +348,8 @@ nautilus_get_icon_file_name (const NautilusIconThemeSpecifications *theme_specs,
 			     const char *name,
 			     const char *modifier,
 			     guint size,
-			     NautilusIconDetails *details)
+			     NautilusIconDetails *details,
+			     NautilusIconDetails *scalable_details)
 {
 	const NautilusIconTheme *theme;
 	char *path;
@@ -349,7 +368,7 @@ nautilus_get_icon_file_name (const NautilusIconThemeSpecifications *theme_specs,
 	/* If there's a modifier, try the modified icon first. */
 	if (modifier != NULL && modifier[0] != '\0') {
 		name_with_modifier = g_strconcat (name, "-", modifier, NULL);
-		path = get_themed_icon_file_path (theme, name_with_modifier, size, details);
+		path = get_themed_icon_file_path (theme, name_with_modifier, size, details, scalable_details);
 		g_free (name_with_modifier);
 		if (path != NULL) {
 			return path;
@@ -357,7 +376,7 @@ nautilus_get_icon_file_name (const NautilusIconThemeSpecifications *theme_specs,
 	}
 	
 	/* Check for a normal icon. */
-	path = get_themed_icon_file_path (theme, name, size, details);
+	path = get_themed_icon_file_path (theme, name, size, details, scalable_details);
 	if (path != NULL) {
 		return path;
 	}
@@ -385,3 +404,59 @@ nautilus_remove_icon_file_name_suffix (const char *icon_name)
 	}
 	return g_strdup (icon_name);
 }
+
+NautilusIconTheme *
+nautilus_icon_theme_new (void)
+{
+	return g_new0 (NautilusIconTheme, 1);
+}
+
+/* utility to check if a theme is in user directory or not */
+static gboolean
+is_theme_in_user_directory (const char *theme_name)
+{
+	char *user_directory, *themes_directory, *this_theme_directory;
+	gboolean result;
+	
+	if (theme_name == NULL) {
+		return FALSE;
+	}
+	
+	user_directory = nautilus_get_user_directory ();
+	themes_directory = g_build_filename (user_directory, "themes", NULL);
+	this_theme_directory = g_build_filename (themes_directory, theme_name, NULL);
+	
+	result = g_file_test (this_theme_directory, G_FILE_TEST_EXISTS);
+	
+	g_free (user_directory);
+	g_free (themes_directory);
+	g_free (this_theme_directory);
+
+	return result;
+}
+
+gboolean
+nautilus_icon_theme_set_names (NautilusIconTheme *icon_theme,
+			       const char        *new_name)
+{
+	g_return_val_if_fail (icon_theme != NULL, FALSE);
+
+	if (eel_strcmp (new_name, icon_theme->name)) {
+		g_free (icon_theme->name);
+		icon_theme->name = g_strdup (new_name);
+		icon_theme->is_in_user_directory = is_theme_in_user_directory (new_name);
+		return TRUE;
+	} else {
+		return FALSE;
+	}
+}
+
+void
+nautilus_icon_theme_destroy (NautilusIconTheme *icon_theme)
+{
+	if (icon_theme) {
+		g_free (icon_theme->name);
+		g_free (icon_theme);
+	}
+}
+
