@@ -38,6 +38,7 @@
 #include <eel/eel-stock-dialogs.h>
 #include <eel/eel-string.h>
 #include <gtk/gtkcellrenderertext.h>
+#include <gtk/gtkhbbox.h>
 #include <gtk/gtkframe.h>
 #include <gtk/gtkhbox.h>
 #include <gtk/gtklabel.h>
@@ -53,6 +54,8 @@
 #include <libgnome/gnome-macros.h>
 #include <libgnomeui/gnome-stock-icons.h>
 #include <libgnomeui/gnome-uidefs.h>
+#include <gtk/gtkmessagedialog.h>
+#include <libgnome/gnome-help.h>
 
 #define RESPONSE_CHOOSE 1000
 
@@ -113,7 +116,7 @@ struct NautilusProgramChooserDetails {
 #define NAME_COLUMN_INITIAL_WIDTH	 200
 
 /* Program name of the mime type capplet */
-#define FILE_TYPES_CAPPLET_NAME 	"gnome2-file-types-properties"
+#define FILE_TYPES_CAPPLET_NAME 	"gnome-file-types-properties"
 
 /* This number controls a maximum character count for a file name that is
  * displayed as part of a dialog (beyond this it will be truncated). 
@@ -161,6 +164,48 @@ program_file_pair_compute_status (ProgramFilePair *pair)
 	return TRUE;
 }
 
+static void
+help_cb (GtkWidget *button, NautilusProgramChooser *program_chooser)
+{
+	GError *error = NULL;
+	gchar *section;
+
+	switch (program_chooser->details->action_type) {
+	case GNOME_VFS_MIME_ACTION_TYPE_APPLICATION:
+		section = "gosnautilus-75";
+		break;
+	case GNOME_VFS_MIME_ACTION_TYPE_COMPONENT:
+	default:
+		section = "gosnautilus-111";
+		break;
+	}
+
+	gnome_help_display_desktop (NULL,
+				    "user-guide",
+				    "wgosnautilus.xml",
+				    section,
+				    &error);
+
+	if (error) {
+		GtkWidget *err_dialog;
+		err_dialog = gtk_message_dialog_new (GTK_WINDOW (gtk_widget_get_toplevel (button)),
+						     GTK_DIALOG_MODAL,
+						     GTK_MESSAGE_ERROR,
+						     GTK_BUTTONS_CLOSE,
+						     _("There was an error displaying help: %s"),
+						     error->message);
+
+		g_signal_connect (G_OBJECT (err_dialog),
+				  "response", G_CALLBACK (gtk_widget_destroy),
+				  NULL);
+
+		gtk_window_set_resizable (GTK_WINDOW (err_dialog), FALSE);
+		gtk_widget_show (err_dialog);
+		g_error_free (error);
+	}
+}
+
+
 static void
 program_file_pair_set_file (ProgramFilePair *pair, NautilusFile *file)
 {
@@ -929,8 +974,8 @@ static void
 launch_mime_capplet_on_ok (GtkDialog *dialog, int response, gpointer callback_data)
 {
 	g_assert (GTK_IS_DIALOG (dialog));
-	
-	if (response == GTK_RESPONSE_OK) {
+
+	if (response == GTK_RESPONSE_YES) {
 		launch_mime_capplet (callback_data);
 	}
 	gtk_object_destroy (GTK_OBJECT (dialog));
@@ -950,7 +995,8 @@ launch_mime_capplet_and_close_dialog (GtkButton *button, gpointer callback_data)
 	
 	g_free (mime_type);
 
-	gtk_object_destroy (GTK_OBJECT (callback_data));
+	gtk_dialog_response (GTK_DIALOG (callback_data),
+		GTK_RESPONSE_DELETE_EVENT);
 }
 
 static void
@@ -1025,13 +1071,13 @@ run_program_configurator_callback (GtkWidget *button, gpointer callback_data)
 	
 
 	/* Radio button for adding to short list for specific file. */
-	radio_button_text = g_strdup_printf (_("Include in the menu just for \"%s\""), 
+	radio_button_text = g_strdup_printf (_("Include in the menu for \"%s\" only"), 
 					     file_name);
 	item_radio_button = pack_radio_button (GTK_BOX (framed_vbox), radio_button_text, type_radio_button);
 	g_free (radio_button_text);
 
 	/* Radio button for setting default for specific file. */
-	radio_button_text = g_strdup_printf (_("Use as default just for \"%s\""), 
+	radio_button_text = g_strdup_printf (_("Use as default for \"%s\" only"), 
 					     file_name);
 	item_default_radio_button = pack_radio_button (GTK_BOX (framed_vbox), radio_button_text, type_radio_button);
 	g_free (radio_button_text);
@@ -1213,15 +1259,12 @@ create_and_set_up_tree_view (NautilusProgramChooser *program_chooser)
 	gtk_tree_view_append_column (GTK_TREE_VIEW (program_chooser->details->tree_view), column);
 
 	/* Update selected item info whenever selection changes. */
-  	g_signal_connect (gtk_tree_view_get_selection (GTK_TREE_VIEW (program_chooser->details->tree_view)),
-			  "changed",
-			  G_CALLBACK (program_list_selection_changed_callback),
-			  program_chooser);
+  	g_signal_connect_object (gtk_tree_view_get_selection (GTK_TREE_VIEW (program_chooser->details->tree_view)),
+				 "changed",
+				 G_CALLBACK (program_list_selection_changed_callback), program_chooser, 0);
 
-	g_signal_connect (program_chooser->details->tree_view,
-			  "row_activated",
-			  G_CALLBACK (tree_view_row_activated_callback),
-			  program_chooser);
+	g_signal_connect_object (program_chooser->details->tree_view, "row_activated",
+				 G_CALLBACK (tree_view_row_activated_callback), program_chooser, 0);
 }
 
 static gboolean
@@ -1252,13 +1295,13 @@ nautilus_program_chooser_finalize (GObject *object)
 	/* Free the ProgramFilePairs */
 	gtk_tree_model_foreach (GTK_TREE_MODEL (program_chooser->details->list_store),
 				tree_model_destroy_program_file_pair, NULL);
-	g_object_unref (G_OBJECT (program_chooser->details->list_store));
+	g_object_unref (program_chooser->details->list_store);
 
 	nautilus_file_unref (program_chooser->details->file);
 
 	g_free (program_chooser->details);
 	
-	EEL_CALL_PARENT (G_OBJECT_CLASS, finalize, (object));
+	G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
 static void
@@ -1276,17 +1319,30 @@ nautilus_program_chooser_instance_init (NautilusProgramChooser *program_chooser)
 {
 	GtkWidget *dialog_vbox, *scrolled_window;
 	GtkWidget *framed_hbox;
+	GtkWidget *help_button;
 	GtkWidget *change_button_holder, *change_button;
 	GtkWidget *capplet_button_frame, *capplet_hbox;
 	GtkWidget *capplet_button, *caption, *capplet_button_vbox;
 	
 	program_chooser->details = g_new0 (NautilusProgramChooserDetails, 1);
 
+	/* This is a slight hack - we add our own help button to the dialog's
+	   button box. We don't want it to go through the normal response
+	   callback. */
+	help_button = gtk_button_new_from_stock (GTK_STOCK_HELP);
+	gtk_box_pack_end (GTK_BOX (GTK_DIALOG (program_chooser)->action_area),
+			  help_button, FALSE, TRUE, 0);
+	gtk_widget_show (help_button);
+	gtk_button_box_set_child_secondary (GTK_BUTTON_BOX (GTK_DIALOG (program_chooser)->action_area), help_button, TRUE);
+
+	g_signal_connect_object (help_button, "clicked",
+				 G_CALLBACK (help_cb), program_chooser, 0);
+
 	program_chooser->details->cancel_button = gtk_dialog_add_button (GTK_DIALOG (program_chooser),
 									 GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL);
 
 	gtk_dialog_add_button (GTK_DIALOG (program_chooser),
-			       _("Choose"), GTK_RESPONSE_OK);
+			       _("C_hoose"), GTK_RESPONSE_OK);
 
 	program_chooser->details->done_button = gtk_dialog_add_button (GTK_DIALOG (program_chooser),
 								       _("Done"), GTK_RESPONSE_CANCEL);
@@ -1297,7 +1353,10 @@ nautilus_program_chooser_instance_init (NautilusProgramChooser *program_chooser)
 	gtk_window_set_default_size (GTK_WINDOW (program_chooser), -1, PROGRAM_CHOOSER_DEFAULT_HEIGHT);
 	gtk_window_set_wmclass (GTK_WINDOW (program_chooser), "program_chooser", "Nautilus");
 
-	dialog_vbox = GTK_DIALOG (program_chooser)->vbox;
+	dialog_vbox = gtk_vbox_new (FALSE, 5);
+	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (program_chooser)->vbox),
+			    dialog_vbox, TRUE, TRUE, 5);
+	gtk_widget_show (dialog_vbox);
 
 	/* Prompt at top of dialog. */
 	program_chooser->details->prompt_label = gtk_label_new (NULL);
@@ -1361,10 +1420,8 @@ nautilus_program_chooser_instance_init (NautilusProgramChooser *program_chooser)
 
   	change_button = gtk_button_new_with_mnemonic (_("_Modify..."));
 
-	g_signal_connect (change_button,
-			  "clicked",
-			  G_CALLBACK (run_program_configurator_callback),
-			  program_chooser);
+	g_signal_connect_object (change_button, "clicked",
+				 G_CALLBACK (run_program_configurator_callback), program_chooser, 0);
 
   	gtk_widget_show (change_button);
   	gtk_box_pack_end (GTK_BOX (change_button_holder), change_button, TRUE, FALSE, 0);
@@ -1384,21 +1441,22 @@ nautilus_program_chooser_instance_init (NautilusProgramChooser *program_chooser)
 	gtk_box_pack_end (GTK_BOX (capplet_hbox), capplet_button_vbox, FALSE, FALSE, 0);
 	capplet_button = gtk_button_new_with_mnemonic (_("_Go There"));	 
 
-	g_signal_connect (capplet_button,
-			  "clicked",
-			  G_CALLBACK (launch_mime_capplet_and_close_dialog),
-			  program_chooser);
+	g_signal_connect_object (capplet_button, "clicked",
+				 G_CALLBACK (launch_mime_capplet_and_close_dialog), program_chooser, 0);
 	gtk_widget_show (capplet_button);
 	gtk_box_pack_start (GTK_BOX (capplet_button_vbox), capplet_button, TRUE, FALSE, 0);
 
 	caption = gtk_label_new (_("You can configure which programs are offered "
-				   "for which file types in the GNOME Control Center."));
+				   "for which file types in the File Types and Programs dialog."));
 	gtk_widget_show (caption);
 	gtk_label_set_line_wrap (GTK_LABEL (caption), TRUE);
 	gtk_box_pack_start (GTK_BOX (capplet_hbox), caption, FALSE, FALSE, 0);				    
 
   	/* Make confirmation button the default. */
   	gtk_dialog_set_default_response (GTK_DIALOG (program_chooser), GTK_RESPONSE_OK);
+
+	/* We don't need the separator as we use frames. */
+  	gtk_dialog_set_has_separator (GTK_DIALOG (program_chooser), FALSE);
 }
 
 
@@ -1422,13 +1480,13 @@ nautilus_program_chooser_new (GnomeVFSMimeActionType action_type,
 
 	switch (action_type) {
 	case GNOME_VFS_MIME_ACTION_TYPE_APPLICATION:
-		title = _("Open with Other");
-		prompt = g_strdup_printf (_("Choose an application with which to open \"%s\"."), file_name);
+		title = _("Open with Other Application");
+		prompt = g_strdup_printf (_("Choose an application with which to open \"%s\":"), file_name);
 		break;
 	case GNOME_VFS_MIME_ACTION_TYPE_COMPONENT:
 	default:
-		title = _("View as Other");
-		prompt = g_strdup_printf (_("Choose a view for \"%s\"."), file_name);
+		title = _("Open with Other Viewer");
+		prompt = g_strdup_printf (_("Choose a view for \"%s\":"), file_name);
 		break;
 	}
 
@@ -1534,8 +1592,8 @@ nautilus_program_chooser_show_no_choices_message (GnomeVFSMimeActionType action_
 		dialog_title = g_strdup (_("No Viewers Available"));
 	} else {
 		g_assert (action_type == GNOME_VFS_MIME_ACTION_TYPE_APPLICATION);
-		unavailable_message = g_strdup_printf (_("No applications are available for \"%s\"."), file_name);
-		dialog_title = g_strdup (_("No Applications Available"));
+		unavailable_message = g_strdup_printf (_("There is no application associated with \"%s\"."), file_name);
+		dialog_title = g_strdup (_("No Application Associated"));
 	}
 
 	/* Note: This might be misleading in the components case, since the
@@ -1543,13 +1601,12 @@ nautilus_program_chooser_show_no_choices_message (GnomeVFSMimeActionType action_
 	 * (They can add applications though.)
 	 */
 	prompt = g_strdup_printf (_("%s\n\n"
-				    "You can configure which programs are offered "
-				    "for which file types with the \"File Types and "
-				    "Programs\" part of the GNOME Control Center. Do "
-				    "you want to go there now?"),
+				    "You can configure GNOME to associate applications "
+				    "with file types.  Do you want to associate an "
+				    "application with this file type now?"),
 				  unavailable_message);
 	dialog = eel_show_yes_no_dialog 
-		(prompt, dialog_title, GTK_STOCK_OK, GTK_STOCK_CANCEL, parent_window);
+		(prompt, dialog_title, _("Associate Application"), GTK_STOCK_CANCEL, parent_window);
 
 	g_signal_connect (dialog, "response",
 			  G_CALLBACK (launch_mime_capplet_on_ok),
