@@ -65,12 +65,16 @@
 #include <sys/stat.h>
 #include <sys/types.h>
 #include <unistd.h>
+#include <gtk/gtkmessagedialog.h>
 
 static const char untranslated_trash_link_name[] = N_("Trash");
 #define TRASH_LINK_NAME _(untranslated_trash_link_name)
 
 #define DESKTOP_COMMAND_EMPTY_TRASH_CONDITIONAL		"/commands/Empty Trash Conditional"
 #define DESKTOP_COMMAND_UNMOUNT_VOLUME_CONDITIONAL	"/commands/Unmount Volume Conditional"
+#define DESKTOP_COMMAND_PROTECT_VOLUME_CONDITIONAL      "/commands/Protect Conditional"
+#define DESKTOP_COMMAND_FORMAT_VOLUME_CONDITIONAL       "/commands/Format Conditional"
+#define DESKTOP_COMMAND_MEDIA_PROPERTIES_VOLUME_CONDITIONAL     "/commands/Media Properties Conditional"
 
 #define DESKTOP_BACKGROUND_POPUP_PATH_DISKS	"/popups/background/Before Zoom Items/Volume Items/Disks"
 
@@ -117,6 +121,8 @@ static gboolean real_supports_zooming                             (FMDirectoryVi
 static void     update_disks_menu                                 (FMDesktopIconView      *view);
 static void     free_volume_black_list                            (FMDesktopIconView      *view);
 static gboolean	volume_link_is_selection 			  (FMDirectoryView 	  *view);
+static NautilusDeviceType volume_link_device_type                 (FMDirectoryView        *view);
+static void     fm_desktop_icon_view_update_icon_container_fonts  (FMDesktopIconView      *view);
 
 EEL_CLASS_BOILERPLATE (FMDesktopIconView,
 			      fm_desktop_icon_view,
@@ -142,9 +148,39 @@ get_icon_container (FMDesktopIconView *icon_view)
 }
 
 static void
-panel_desktop_area_changed (FMDesktopIconView *icon_view)
+icon_container_set_workarea (NautilusIconContainer *icon_container,
+			     long                  *workareas,
+			     int                    n_items)
 {
-	long *borders = NULL;
+	int left, right, top, bottom;
+	int screen_width, screen_height;
+	int i;
+
+	left = right = top = bottom = 0;
+
+	screen_width  = gdk_screen_width ();
+	screen_height = gdk_screen_height ();
+
+	for (i = 0; i < n_items; i += 4) {
+		int x      = workareas [i];
+		int y      = workareas [i + 1];
+		int width  = workareas [i + 2];
+		int height = workareas [i + 3];
+
+		left   = MAX (left, x);
+		right  = MAX (right, screen_width - width - x);
+		top    = MAX (top, y);
+		bottom = MAX (bottom, screen_height - height - y);
+	}
+
+	nautilus_icon_container_set_margins (icon_container,
+					     left, right, top, bottom);
+}
+
+static void
+net_workarea_changed (FMDesktopIconView *icon_view)
+{
+	long *workareas = NULL;
 	Atom type_returned;
 	int format_returned;
 	unsigned long items_returned;
@@ -158,38 +194,33 @@ panel_desktop_area_changed (FMDesktopIconView *icon_view)
 	gdk_error_trap_push ();
 	if (XGetWindowProperty (GDK_DISPLAY (),
 				GDK_ROOT_WINDOW (),
-				gdk_x11_get_xatom_by_name ("GNOME_PANEL_DESKTOP_AREA"),
-				0 /* long_offset */, 
-				4 /* long_length */,
-				False /* delete */,
+				gdk_x11_get_xatom_by_name ("_NET_WORKAREA"),
+				0, G_MAXLONG, False,
 				XA_CARDINAL,
 				&type_returned,
 				&format_returned,
 				&items_returned,
 				&bytes_after_return,
-				(unsigned char **)&borders) != Success) {
-		if (borders != NULL)
-			XFree (borders);
-		borders = NULL;
+				(unsigned char **)&workareas) != Success) {
+		if (workareas != NULL)
+			XFree (workareas);
+		workareas = NULL;
 	}
 			    
 	if (gdk_error_trap_pop ()
-	    || borders == NULL
+	    || workareas == NULL
 	    || type_returned != XA_CARDINAL
-	    || items_returned != 4
+	    || (items_returned % 4) != 0
 	    || format_returned != 32) {
 		nautilus_icon_container_set_margins (icon_container,
 						     0, 0, 0, 0);
 	} else {
-		nautilus_icon_container_set_margins (icon_container,
-						     borders[0 /* left */],
-						     borders[1 /* right */],
-						     borders[2 /* top */],
-						     borders[3 /* bottom */]);
+		icon_container_set_workarea (
+				icon_container, workareas, items_returned);
 	}
 
-	if (borders != NULL)
-		XFree (borders);
+	if (workareas != NULL)
+		XFree (workareas);
 }
 
 static GdkFilterReturn
@@ -204,9 +235,8 @@ desktop_icon_view_property_filter (GdkXEvent *gdk_xevent,
   
 	switch (xevent->type) {
 	case PropertyNotify:
-		if (xevent->xproperty.atom == gdk_x11_get_xatom_by_name ("GNOME_PANEL_DESKTOP_AREA")) {
-			panel_desktop_area_changed (icon_view);
-		}
+		if (xevent->xproperty.atom == gdk_x11_get_xatom_by_name ("_NET_WORKAREA"))
+			net_workarea_changed (icon_view);
 		break;
 	default:
 		break;
@@ -423,23 +453,29 @@ create_mount_link (FMDesktopIconView *icon_view,
 	case NAUTILUS_DEVICE_NFS:
 		icon_name = "i-nfs";
 		break;
+
+	case NAUTILUS_DEVICE_SMB:
+		icon_name = "i-smb";
+		break;
 	
 	case NAUTILUS_DEVICE_ZIP_DRIVE:
 		icon_name = "i-zipdisk";
 		break;
 
+	case NAUTILUS_DEVICE_APPLE:
+	case NAUTILUS_DEVICE_WINDOWS:
 	case NAUTILUS_DEVICE_CAMERA:
 	case NAUTILUS_DEVICE_UNKNOWN:
 		break;
 	}
 
 	target_uri = nautilus_volume_get_target_uri (volume);
-	
+
 	volume_name = create_unique_volume_name (volume);
-	
+
 	/* Create link */
 	nautilus_link_local_create (desktop_directory, volume_name, icon_name, target_uri, NULL, NAUTILUS_LINK_MOUNT);
-				    
+
 	g_free (target_uri);
 	g_free (volume_name);
 }
@@ -554,7 +590,29 @@ delayed_init (FMDesktopIconView *desktop_icon_view)
 	desktop_icon_view->details->delayed_init_signal = 0;
 }
 
+static void
+font_changed_callback (gpointer callback_data)
+{
+ 	g_return_if_fail (FM_IS_DESKTOP_ICON_VIEW (callback_data));
+	
+	fm_desktop_icon_view_update_icon_container_fonts (FM_DESKTOP_ICON_VIEW (callback_data));
+}
 
+static void
+fm_desktop_icon_view_update_icon_container_fonts (FMDesktopIconView *icon_view)
+{
+	NautilusIconContainer *icon_container;
+	char *font;
+	
+	icon_container = get_icon_container (icon_view);
+	g_assert (icon_container != NULL);
+
+	font = eel_preferences_get (NAUTILUS_PREFERENCES_DESKTOP_FONT);
+
+	nautilus_icon_container_set_font (icon_container, font);
+
+	g_free (font);
+}
 
 static void
 fm_desktop_icon_view_init (FMDesktopIconView *desktop_icon_view)
@@ -573,6 +631,7 @@ fm_desktop_icon_view_init (FMDesktopIconView *desktop_icon_view)
 
 	icon_container = get_icon_container (desktop_icon_view);
 
+	nautilus_icon_container_set_use_drop_shadows (icon_container, TRUE);
 	fm_icon_container_set_sort_desktop (FM_ICON_CONTAINER (icon_container), TRUE);
 
 	/* Set up details */
@@ -641,13 +700,20 @@ fm_desktop_icon_view_init (FMDesktopIconView *desktop_icon_view)
 	eel_preferences_add_callback (NAUTILUS_PREFERENCES_ICON_VIEW_DEFAULT_ZOOM_LEVEL,
 				      default_zoom_level_changed,
 				      desktop_icon_view);
+	
+	eel_preferences_add_callback_while_alive (NAUTILUS_PREFERENCES_DESKTOP_FONT,
+						  font_changed_callback, 
+						  desktop_icon_view, G_OBJECT (desktop_icon_view));
+	
 	default_zoom_level_changed (desktop_icon_view);
+	fm_desktop_icon_view_update_icon_container_fonts (desktop_icon_view);
 
-	/* Read out the panel desktop area and update the icon container
-	 * accordingly */
-	panel_desktop_area_changed (desktop_icon_view);
+	/* Read out the workarea geometry and update the icon container accordingly */
+	net_workarea_changed (desktop_icon_view);
 
 	/* Setup the property filter */
+	XSelectInput (GDK_DISPLAY (), GDK_ROOT_WINDOW (), PropertyChangeMask);
+
 	gdk_window_add_filter (gdk_get_default_root_window (),
 			       desktop_icon_view_property_filter,
 			       desktop_icon_view);
@@ -703,40 +769,166 @@ reset_background_callback (BonoboUIComponent *component,
 		(fm_directory_view_get_background (FM_DIRECTORY_VIEW (data)));
 }
 
+static gboolean
+have_volume_format_app (void)
+{
+	static int have_app = -1;
+	char *app;
+
+	if (have_app < 0) {
+		app = g_find_program_in_path ("gmedia_format");
+		if (app == NULL) {
+			app = g_find_program_in_path ("gfloppy");
+		}
+		have_app = (app != NULL);
+		g_free (app);
+	}
+	return have_app;
+}
+
+static gboolean
+have_volume_properties_app (void)
+{
+	static int have_app = -1;
+	char *app;
+
+	if (have_app < 0) {
+		app = g_find_program_in_path ("gmedia_prop");
+		have_app = (app != NULL);
+		g_free (app);
+	}
+	return have_app;
+}
+
+static gboolean
+have_volume_protection_app (void)
+{
+	static int have_app = -1;
+	char *app;
+
+	if (have_app < 0) {
+		app = g_find_program_in_path ("gmedia_prot");
+		have_app = (app != NULL);
+		g_free (app);
+	}
+	return have_app;
+}
+
 static void
-unmount_volume_callback (BonoboUIComponent *component, gpointer data, const char *verb)
+volume_ops_callback (BonoboUIComponent *component, gpointer data, const char *verb)
 {
         FMDirectoryView *view;
 	NautilusFile *file;
-	char *uri, *path, *mount_uri, *mount_path;
+	char *uri, *mount_uri, *mount_path;
 	GList *selection;
-			    
+	char *command;
+	const char *device_path;
+	char *rawdevice_path, *quoted_path;
+	char *program;
+	NautilusVolume *volume;
+	gboolean status;
+	GError *error;
+	GtkWidget *dialog;
+
+
         g_assert (FM_IS_DIRECTORY_VIEW (data));
         
         view = FM_DIRECTORY_VIEW (data);
         
-       if (!volume_link_is_selection (view)) {
+	if (!volume_link_is_selection (view)) {
 		return;       
-       }
+	}
               
 	selection = fm_directory_view_get_selection (view);
-
+	
 	file = NAUTILUS_FILE (selection->data);
-	uri = nautilus_file_get_uri (file);
-	path = gnome_vfs_get_local_path_from_uri (uri);
-	if (path != NULL) {
-		mount_uri = nautilus_link_local_get_link_uri (path);
-		mount_path = gnome_vfs_get_local_path_from_uri (mount_uri);
-		if (mount_path != NULL) {
-			nautilus_volume_monitor_mount_unmount_removable (nautilus_volume_monitor_get (), mount_path, FALSE);
-		}
-		
-		g_free (mount_path);
-		g_free (mount_uri);
-		g_free (path);
+	
+	if (!nautilus_file_is_local (file)) {
+		nautilus_file_list_free (selection);
+		return;
 	}
+		
+	uri = nautilus_file_get_uri (file);
+	mount_uri = nautilus_link_local_get_link_uri (uri);
+	mount_path = gnome_vfs_get_local_path_from_uri (mount_uri);
 	g_free (uri);
+	g_free (mount_uri);
+	if (mount_path == NULL) {
+		nautilus_file_list_free (selection);
+		return;
+	}
 
+	volume = nautilus_volume_monitor_get_volume_for_path (nautilus_volume_monitor_get (), mount_path);
+	device_path = nautilus_volume_get_device_path (volume);
+	if (device_path == NULL) {
+		g_free (mount_path);
+		nautilus_file_list_free (selection);
+		return;
+	}
+		
+	/* Solaris specif cruft: */
+	if (eel_str_has_prefix (device_path, "/vol/dev/")) {
+		rawdevice_path = g_strconcat ("/vol/dev/r",
+					device_path + strlen ("/vol/dev/"),
+					NULL);
+	} else {
+		rawdevice_path = g_strdup (device_path);
+	}
+	
+	quoted_path = g_shell_quote (rawdevice_path);
+	g_free (rawdevice_path);
+	rawdevice_path = quoted_path;
+		
+	if (strcmp (verb, "Unmount Volume Conditional") == 0) {
+		nautilus_volume_monitor_mount_unmount_removable (nautilus_volume_monitor_get (),
+								 mount_path, FALSE, TRUE);
+	} else {
+		command = NULL;
+		
+		if (strcmp (verb, "Format Conditional") == 0) {
+			program = g_find_program_in_path ("gmedia_format");
+			if (program != NULL) {
+				command = g_strdup_printf ("%s -d %s", program, rawdevice_path);
+				g_free (program);
+			} else {
+				program = g_find_program_in_path ("gfloppy");
+				if (program != NULL) {
+					command = g_strdup_printf ("%s --device %s", program, device_path);
+					g_free (program);
+				}
+			}
+		} else if (strcmp (verb, "Media Properties Conditional") == 0) {
+			program = g_find_program_in_path ("gmedia_prop");
+			if (program) {
+				command = g_strdup_printf ("%s %s", program, rawdevice_path);
+				g_free (program);
+			} 
+		} else if (strcmp (verb, "Protect Conditional") == 0) {
+			program = g_find_program_in_path ("gmedia_prot");
+			if (program) {
+				command = g_strdup_printf ("%s %s", program, rawdevice_path);
+				g_free (program);
+			} 
+		}
+
+		if (command) {
+			error = NULL;
+			status = g_spawn_command_line_async (command, &error);
+			if (!status) {
+				dialog = gtk_message_dialog_new (NULL, 0,
+								 GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE, 
+								 _("Error executing utility program '%s': %s"), command, error->message);
+				g_signal_connect (G_OBJECT (dialog), "response", 
+						  G_CALLBACK (gtk_widget_destroy), NULL);
+				gtk_widget_show (dialog);
+				g_error_free (error);
+			}
+			g_free (command);
+		}
+	}
+	
+	g_free (rawdevice_path);
+	g_free (mount_path);
 	nautilus_file_list_free (selection);
 }
 
@@ -745,7 +937,7 @@ trash_link_is_selection (FMDirectoryView *view)
 {
 	GList *selection;
 	gboolean result;
-	char *uri, *path;
+	char *uri;
 
 	result = FALSE;
 	
@@ -757,11 +949,9 @@ trash_link_is_selection (FMDirectoryView *view)
 		/* It's probably OK that this only works for local
 		 * items, since the trash we care about is on the desktop.
 		 */
-		path = gnome_vfs_get_local_path_from_uri (uri);
-		if (path != NULL && nautilus_link_local_is_trash_link (path)) {
+		if (nautilus_link_local_is_trash_link (uri)) {
 			result = TRUE;
 		}
-		g_free (path);
 		g_free (uri);
 	}
 	
@@ -775,7 +965,7 @@ volume_link_is_selection (FMDirectoryView *view)
 {
 	GList *selection;
 	gboolean result;
-	char *uri, *path;
+	char *uri;
 
 	result = FALSE;
 	
@@ -787,11 +977,9 @@ volume_link_is_selection (FMDirectoryView *view)
 		/* It's probably OK that this only works for local
 		 * items, since the volume we care about is on the desktop.
 		 */
-		path = gnome_vfs_get_local_path_from_uri (uri);
-		if (path != NULL && nautilus_link_local_is_volume_link (path)) {
+		if (nautilus_link_local_is_volume_link (uri)) {
 			result = TRUE;
 		}
-		g_free (path);
 		g_free (uri);
 	}
 	
@@ -800,6 +988,43 @@ volume_link_is_selection (FMDirectoryView *view)
 	return result;
 }
 
+/*
+ *  Returns Device Type for device icon on desktop
+ */
+
+static NautilusDeviceType
+volume_link_device_type (FMDirectoryView *view)
+{
+	GList *selection;
+	gchar *uri, *mount_uri, *mount_path;
+	NautilusVolume *volume;
+
+	selection = fm_directory_view_get_selection (view);
+
+	if (selection == NULL) {
+		return NAUTILUS_DEVICE_UNKNOWN;
+	}
+
+	volume = NULL;
+	
+	uri = nautilus_file_get_uri (NAUTILUS_FILE (selection->data));
+	mount_uri = nautilus_link_local_get_link_uri (uri);
+	mount_path = gnome_vfs_get_local_path_from_uri (mount_uri);
+	if(mount_path != NULL) {
+		volume = nautilus_volume_monitor_get_volume_for_path (nautilus_volume_monitor_get (), mount_path);
+		g_free (mount_path);
+	}
+	g_free (mount_uri);
+	g_free (uri);
+	nautilus_file_list_free (selection);
+
+	if (volume != NULL)
+		return nautilus_volume_get_device_type (volume);
+
+	return NAUTILUS_DEVICE_UNKNOWN;
+}
+
+
 static void
 fm_desktop_icon_view_trash_state_changed_callback (NautilusTrashMonitor *trash_monitor,
 						   gboolean state,
@@ -807,7 +1032,7 @@ fm_desktop_icon_view_trash_state_changed_callback (NautilusTrashMonitor *trash_m
 {
 	char *path;
 
-	path = nautilus_make_path (desktop_directory, TRASH_LINK_NAME);
+	path = g_build_filename (desktop_directory, TRASH_LINK_NAME, NULL);
 
 	nautilus_link_local_set_icon (path, state ? "trash-empty" : "trash-full");
 
@@ -855,7 +1080,7 @@ volume_unmounted_callback (NautilusVolumeMonitor *monitor,
 		return;
 	}
 	
-	link_path = nautilus_make_path (desktop_directory, volume_name);
+	link_path = g_build_filename (desktop_directory, volume_name, NULL);
 	unlink_and_notify (link_path);
 
 	g_free (volume_name);
@@ -890,7 +1115,7 @@ update_link_and_delete_copies (gboolean (*is_link_function) (const char *path),
 	found_link = FALSE;
 
 	while ((dir_entry = readdir (dir)) != NULL) {
-		link_path = nautilus_make_path (desktop_directory, dir_entry->d_name);
+		link_path = g_build_filename (desktop_directory, dir_entry->d_name, NULL);
 		if ((* is_link_function) (link_path)) {
 			if (!found_link &&
 			     (link_name == NULL || strcmp (dir_entry->d_name, link_name) == 0)) {
@@ -928,7 +1153,11 @@ update_home_link_and_delete_copies (void)
 	 */
 	home_link_name = g_strdup_printf (_("%s's Home"), g_get_user_name ());
 	
+#ifdef WEB_NAVIGATION_ENABLED
 	home_uri = eel_preferences_get (NAUTILUS_PREFERENCES_HOME_URI);
+#else
+	home_uri = gnome_vfs_get_uri_from_local_path (g_get_home_dir ());
+#endif
 	
 	if (!update_link_and_delete_copies (nautilus_link_local_is_home_link,
 					    NULL,
@@ -985,7 +1214,6 @@ mount_parameters_new (FMDesktopIconView *view, const char *mount_path)
 	g_assert (!eel_str_is_empty (mount_path)); 
 
 	new_parameters = g_new (MountParameters, 1);
-	g_object_ref (view);
 	new_parameters->view = view;
 	new_parameters->mount_path = g_strdup (mount_path);
 
@@ -997,7 +1225,6 @@ mount_parameters_free (MountParameters *parameters)
 {
 	g_assert (parameters != NULL);
 
-	g_object_unref (parameters->view);
 	g_free (parameters->mount_path);
 	g_free (parameters);
 }
@@ -1016,6 +1243,7 @@ mount_or_unmount_removable_volume (BonoboUIComponent *component,
 	       			   gpointer user_data)
 {
 	MountParameters *parameters;
+	gboolean mount;
 
 	g_assert (BONOBO_IS_UI_COMPONENT (component));
 
@@ -1025,10 +1253,11 @@ mount_or_unmount_removable_volume (BonoboUIComponent *component,
 	}
 
 	parameters = (MountParameters *) user_data;
+	mount = (strcmp (state, "1") == 0);
 	nautilus_volume_monitor_mount_unmount_removable 
 		(nautilus_volume_monitor_get (),
 		 parameters->mount_path,
-		 strcmp (state, "1") == 0);
+		 mount, !mount);
 	update_disks_menu (parameters->view);
 }	       
 
@@ -1093,7 +1322,9 @@ real_update_menus (FMDirectoryView *view)
 {
 	FMDesktopIconView *desktop_view;
 	char *label;
-	gboolean include_empty_trash, include_unmount_volume;
+	gboolean include_empty_trash, include_media_commands;
+	NautilusDeviceType media_type;
+	char *unmount_label;
 	
 	g_assert (FM_IS_DESKTOP_ICON_VIEW (view));
 
@@ -1120,11 +1351,7 @@ real_update_menus (FMDirectoryView *view)
 		 DESKTOP_COMMAND_EMPTY_TRASH_CONDITIONAL,
 		 !include_empty_trash);
 	if (include_empty_trash) {
-		if (eel_preferences_get_boolean (NAUTILUS_PREFERENCES_CONFIRM_TRASH)) {
-			label = g_strdup (_("Empty Trash..."));
-		} else {
-			label = g_strdup (_("Empty Trash"));
-		}
+		label = g_strdup (_("Empty Trash"));
 		nautilus_bonobo_set_label
 			(desktop_view->details->ui, 
 			 DESKTOP_COMMAND_EMPTY_TRASH_CONDITIONAL,
@@ -1137,23 +1364,150 @@ real_update_menus (FMDirectoryView *view)
 	}
 
 	/* Unmount Volume */
-	include_unmount_volume = volume_link_is_selection (view);
-	nautilus_bonobo_set_hidden
-		(desktop_view->details->ui,
-		 DESKTOP_COMMAND_UNMOUNT_VOLUME_CONDITIONAL,
-		 !include_unmount_volume);
-	if (include_unmount_volume) {
-		label = g_strdup (_("Unmount Volume"));
+	include_media_commands = volume_link_is_selection (view);
+
+	if (include_media_commands) {
+		media_type = volume_link_device_type (view);
+
+		unmount_label = _("E_ject");
+		
+		switch(media_type) {
+		case NAUTILUS_DEVICE_FLOPPY_DRIVE:
+			if (have_volume_format_app ()) {
+				nautilus_bonobo_set_hidden
+					(desktop_view->details->ui,
+					DESKTOP_COMMAND_FORMAT_VOLUME_CONDITIONAL, FALSE);
+				nautilus_bonobo_set_sensitive
+					(desktop_view->details->ui,
+					DESKTOP_COMMAND_FORMAT_VOLUME_CONDITIONAL, TRUE);
+			} else {
+				nautilus_bonobo_set_hidden
+					(desktop_view->details->ui,
+					DESKTOP_COMMAND_FORMAT_VOLUME_CONDITIONAL, TRUE);
+			}
+
+			if (have_volume_properties_app ()) {
+				nautilus_bonobo_set_hidden
+					(desktop_view->details->ui,
+					 DESKTOP_COMMAND_MEDIA_PROPERTIES_VOLUME_CONDITIONAL, FALSE);
+				nautilus_bonobo_set_sensitive
+					(desktop_view->details->ui,
+			 		DESKTOP_COMMAND_MEDIA_PROPERTIES_VOLUME_CONDITIONAL, TRUE);
+			} else {
+				nautilus_bonobo_set_hidden
+					(desktop_view->details->ui,
+					 DESKTOP_COMMAND_MEDIA_PROPERTIES_VOLUME_CONDITIONAL, TRUE);
+			}
+
+			nautilus_bonobo_set_hidden
+				(desktop_view->details->ui,
+				DESKTOP_COMMAND_PROTECT_VOLUME_CONDITIONAL, TRUE);
+			break;
+		
+		case NAUTILUS_DEVICE_CDROM_DRIVE:
+			nautilus_bonobo_set_hidden
+				(desktop_view->details->ui,
+				DESKTOP_COMMAND_PROTECT_VOLUME_CONDITIONAL, TRUE);
+
+			nautilus_bonobo_set_hidden
+				(desktop_view->details->ui,
+				DESKTOP_COMMAND_FORMAT_VOLUME_CONDITIONAL, TRUE);
+
+			if (have_volume_properties_app ()) {
+				nautilus_bonobo_set_hidden
+					(desktop_view->details->ui,
+					DESKTOP_COMMAND_MEDIA_PROPERTIES_VOLUME_CONDITIONAL, FALSE);
+				nautilus_bonobo_set_sensitive
+					(desktop_view->details->ui,
+					DESKTOP_COMMAND_MEDIA_PROPERTIES_VOLUME_CONDITIONAL, TRUE);
+			} else {
+				nautilus_bonobo_set_hidden
+					(desktop_view->details->ui,
+					DESKTOP_COMMAND_MEDIA_PROPERTIES_VOLUME_CONDITIONAL, TRUE);
+			}
+			break;	
+		
+		case NAUTILUS_DEVICE_ZIP_DRIVE:
+		case NAUTILUS_DEVICE_JAZ_DRIVE:
+			if (have_volume_format_app ()) {
+				nautilus_bonobo_set_hidden
+					(desktop_view->details->ui,
+					 DESKTOP_COMMAND_FORMAT_VOLUME_CONDITIONAL, FALSE);
+				nautilus_bonobo_set_sensitive
+					(desktop_view->details->ui,
+					DESKTOP_COMMAND_FORMAT_VOLUME_CONDITIONAL, TRUE);
+			} else {
+				nautilus_bonobo_set_hidden
+					(desktop_view->details->ui,
+					 DESKTOP_COMMAND_FORMAT_VOLUME_CONDITIONAL, TRUE);
+			}
+
+			if (have_volume_properties_app ()) {
+				nautilus_bonobo_set_hidden
+					(desktop_view->details->ui,
+					DESKTOP_COMMAND_MEDIA_PROPERTIES_VOLUME_CONDITIONAL, FALSE);
+				nautilus_bonobo_set_sensitive
+					(desktop_view->details->ui,
+					 DESKTOP_COMMAND_MEDIA_PROPERTIES_VOLUME_CONDITIONAL, TRUE);
+			} else {
+				nautilus_bonobo_set_hidden
+					(desktop_view->details->ui,
+					DESKTOP_COMMAND_MEDIA_PROPERTIES_VOLUME_CONDITIONAL, TRUE);
+			}
+
+			if (have_volume_protection_app ()) {
+				nautilus_bonobo_set_hidden
+					(desktop_view->details->ui,
+					 DESKTOP_COMMAND_PROTECT_VOLUME_CONDITIONAL, FALSE);
+				nautilus_bonobo_set_sensitive
+					(desktop_view->details->ui,
+					 DESKTOP_COMMAND_PROTECT_VOLUME_CONDITIONAL, TRUE);
+			} else {
+				nautilus_bonobo_set_hidden
+					(desktop_view->details->ui,
+					 DESKTOP_COMMAND_PROTECT_VOLUME_CONDITIONAL, TRUE);
+			}
+			break;
+		default:
+			unmount_label = _("_Unmount Volume");
+			break;
+		}
+
+		/* We always want a unmount entry */
+		nautilus_bonobo_set_hidden
+			(desktop_view->details->ui,
+			 DESKTOP_COMMAND_UNMOUNT_VOLUME_CONDITIONAL,
+			 FALSE);
+		nautilus_bonobo_set_sensitive
+			(desktop_view->details->ui,
+			DESKTOP_COMMAND_UNMOUNT_VOLUME_CONDITIONAL, TRUE);
+
+		/* But call it eject for removable media */
 		nautilus_bonobo_set_label
 			(desktop_view->details->ui, 
+			 DESKTOP_COMMAND_UNMOUNT_VOLUME_CONDITIONAL, unmount_label);
+	} else {
+		nautilus_bonobo_set_hidden
+			(desktop_view->details->ui,
+			 DESKTOP_COMMAND_PROTECT_VOLUME_CONDITIONAL,
+			 TRUE);
+		
+		nautilus_bonobo_set_hidden
+			(desktop_view->details->ui,
+			 DESKTOP_COMMAND_FORMAT_VOLUME_CONDITIONAL,
+			 TRUE);
+		
+		nautilus_bonobo_set_hidden
+			(desktop_view->details->ui,
+			 DESKTOP_COMMAND_MEDIA_PROPERTIES_VOLUME_CONDITIONAL,
+			 TRUE);
+		
+		nautilus_bonobo_set_hidden
+			(desktop_view->details->ui,
 			 DESKTOP_COMMAND_UNMOUNT_VOLUME_CONDITIONAL,
-			 label);
-		nautilus_bonobo_set_sensitive 
-			(desktop_view->details->ui, 
-			 DESKTOP_COMMAND_UNMOUNT_VOLUME_CONDITIONAL, TRUE);
-		g_free (label);
+			 TRUE);
 	}
-
+	
 	bonobo_ui_component_thaw (desktop_view->details->ui, NULL);
 }
 
@@ -1166,9 +1520,12 @@ real_merge_menus (FMDirectoryView *view)
 		BONOBO_UI_VERB ("Change Background", change_background_callback),
 		BONOBO_UI_VERB ("Empty Trash Conditional", empty_trash_callback),
 		BONOBO_UI_VERB ("New Terminal", new_terminal_callback),
-		BONOBO_UI_VERB ("New Launcher", new_launcher_callback),
+		BONOBO_UI_VERB ("New Launcher Desktop", new_launcher_callback),
 		BONOBO_UI_VERB ("Reset Background", reset_background_callback),
-		BONOBO_UI_VERB ("Unmount Volume Conditional", unmount_volume_callback),
+		BONOBO_UI_VERB ("Unmount Volume Conditional", volume_ops_callback),
+		BONOBO_UI_VERB ("Protect Conditional", volume_ops_callback),
+		BONOBO_UI_VERB ("Format Conditional", volume_ops_callback),
+		BONOBO_UI_VERB ("Media Properties Conditional", volume_ops_callback),
 		BONOBO_UI_VERB_END
 	};
 
